/* Epiphany G3/G4 Hardware Description */
#ifndef _EPIPHANY_G34_H_
#define _EPIPHANY_G34_H_

#include <stdint.h>

/* MOVTS/MOVFS and TRAP wrappers */
#define MOVFS(REG) \
	({ uint32_t r; __asm__ volatile ("movfs %0, "#REG : "=r"(r)); r; })
#define MOVTS(REG,VALUE) \
	do { __asm__ volatile ("movts "#REG", %0" : : "r"(VALUE)); } while(0);
#define DO_TRAP(num) do { __asm__ volatile ("TRAP "#num); } while(0);
#define TRAP(num) DO_TRAP(num)

/* Register Type Definitions */
typedef       uint32_t rw;
typedef       uint32_t wo;
typedef const uint32_t ro;
typedef const uint32_t un;

/* CORE Registers (CORE) */
struct __attribute__((packed,aligned(8))) CORE_s {
	rw CONFIG;	/* NOTE: must be accessed with MOVFS/MOVTS */
	rw STATUS;	/* NOTE: must be accessed with MOVFS/MOVTS */
	rw PC;
	ro DEBUGSTATUS;
	un Res0[1];	/* IAB */
	rw LC;
	rw LS;
	rw LE;
	rw IRET;
	rw IMASK;
	rw ILAT;
	wo ILATST;
	wo ILATCL;
	rw IPEND;
	rw CTIMER0;
	rw CTIMER1;
	wo FSTATUS;
	un Res1[0];
	wo DEBUGCMD;
};

/* DMA Registers (DMA) */
struct __attribute__((packed,aligned(8))) DMA_s {
	rw CONFIG;
	rw STRIDE;
	rw COUNT;
	rw SRCADDR;
	rw DSTADDR;
	rw AUTO0;
	ro AUTO1;
	rw STATUS;
};

/* "Processor Control" == Memory System Registers (MEM) */
struct __attribute__((packed,aligned(8))) MEM_s {
	un Res0[1];
	rw STATUS;
	rw PROTECT;
};

/* Mesh Node Control Registers (MESH) */
struct __attribute__((packed,aligned(8))) MESH_s {
	rw MESHCONFIG;
	ro COREID;
	rw MULTICAST;
	wo RESETCORE;
	rw CMESHROUTE;
	rw XMESHROUTE;
	rw RMESHROUTE;
};

/* initialized in startup code */
extern volatile struct CORE_s* CORE;
extern volatile struct DMA_s*  DMA0;
extern volatile struct DMA_s*  DMA1;
extern volatile struct MEM_s*  MEM;
extern volatile struct MESH_s* MESH;

#endif /* _EPIPHANY_G34_H_ */
