/* Linker script for Epiphany G3/G4 */
MEMORY {
	SRAM_FULL  (rwx) : ORIGIN = 0x0000, LENGTH = 32k

	SRAM_BANK0 (rwx) : ORIGIN = 0x0000, LENGTH = 8k
	SRAM_BANK1 (rwx) : ORIGIN = 0x2000, LENGTH = 8k
	SRAM_BANK2 (rwx) : ORIGIN = 0x4000, LENGTH = 8k
	SRAM_BANK3 (rwx) : ORIGIN = 0x6000, LENGTH = 8k

	DRAM_BANK0 (rwx) : ORIGIN = 0x8e000000, LENGTH = 16M
	DRAM_BANK1 (rwx) : ORIGIN = 0x8f000000, LENGTH = 16M
}

PROVIDE_HIDDEN (_stack = ORIGIN(SRAM_FULL) + LENGTH(SRAM_FULL));
OUTPUT_FORMAT("elf32-epiphany", "elf32-epiphany", "elf32-epiphany")
OUTPUT_ARCH(epiphany)
EXTERN (Vector_Table)
EXTERN (Default_Handler)
ENTRY  (Reset_Handler)

SECTIONS {
	/* ABI-reserved area */
	.vectors      0x00 : { KEEP(*(.vectors))                 } >SRAM_FULL
	workgroup_cfg 0x28 : { KEEP(*(.workgroup_cfg)) . = 0x28; } >SRAM_FULL
	ext_mem_cfg   0x50 : { KEEP(*(.ext_mem_cfg))   . = 0x8;  } >SRAM_FULL
	loader_cfg    0x58 : { KEEP(*(.loader_cfg))    . = 0x10; } >SRAM_FULL
	.reserved     0x68 : { KEEP(*(.reserved))      . = 0x98; } >SRAM_FULL

	/* manual assignments */
	.bank0       : { *(.bank0 .text_bank0) *(.data_bank0) } >SRAM_BANK0
	.bank1       : { *(.bank1 .text_bank1) *(.data_bank1) } >SRAM_BANK1
	.bank2       : { *(.bank2 .text_bank2) *(.data_bank2) } >SRAM_BANK2
	.bank3       : { *(.bank3 .text_bank3) *(.data_bank3) } >SRAM_BANK3
	.code_dram   : { *(.code_dram  ) } >DRAM_BANK0
	.shared_dram : { *(.shared_dram) } >DRAM_BANK1

	/* constructors/destructors */
	.preinit_array : {
		. = ALIGN(8);
		PROVIDE_HIDDEN (__preinit_array_start = .);
		KEEP (*(.preinit_array))
		PROVIDE_HIDDEN (__preinit_array_end = .);
	} >SRAM_FULL
	.init_array : {
		. = ALIGN(8);
		PROVIDE_HIDDEN (__init_array_start = .);
		KEEP (*(SORT(.init_array.*)))
		KEEP (*(.init_array))
		PROVIDE_HIDDEN (__init_array_end = .);
	} >SRAM_FULL
	.fini_array : {
		. = ALIGN(8);
		PROVIDE_HIDDEN (__fini_array_start = .);
		KEEP (*(.fini_array))
		KEEP (*(SORT(.fini_array.*)))
		PROVIDE_HIDDEN (__fini_array_end = .);
	} >SRAM_FULL

	/* avoid crt{begin,i,n,end}.o need */
	.ctors : {
		. = ALIGN(8);
		PROVIDE_HIDDEN (__ctors_start = .);
		KEEP (*(SORT(.ctors.*)))
		KEEP (*(.ctors))
		PROVIDE_HIDDEN (__ctors_end = .);
	} >SRAM_FULL
	.dtors : {
		. = ALIGN(8);
		PROVIDE_HIDDEN (__dtors_start = .);
		KEEP (*(SORT(.dtors.*)))
		KEEP (*(.dtors))
		PROVIDE_HIDDEN (__dtors_end = .);
	} >SRAM_FULL

	/* text segment */
	.text : {
		*(.vectors)
		*(.text*)
		*(.rodata*)
		. = ALIGN(8);
	} >SRAM_FULL
	_etext = .;

	/* data segment */
	.data : {
		PROVIDE_HIDDEN (_sdata = .);
		*(.data*)
		. = ALIGN(8);
		PROVIDE_HIDDEN (_edata = .);
	} >SRAM_FULL
	. = ALIGN(8);

	/* zero-initialized data */
	.bss (NOLOAD) : {
		PROVIDE_HIDDEN (_sbss = .);
		*(.bss)
		*(COMMON)
		. = ALIGN(8);
		PROVIDE_HIDDEN (_ebss = .);
	} >SRAM_FULL

	/* might be needed for C++ exception handling */
	/DISCARD/ : { *(.eh_frame) }

	. = ALIGN(4);
	_end = .;
}

