/* Startup Code for Epiphany G3/G4 */
#include <stdint.h>
#include <stdlib.h>

#include <epiphany-g34.h>

/* Global Variables */
volatile struct CORE_s* CORE;
volatile struct DMA_s*  DMA0;
volatile struct DMA_s*  DMA1;
volatile struct MEM_s*  MEM;
volatile struct MESH_s* MESH;

/* Interrupt Handlers */
#define DEFAULT_HANDLER(SOURCE) __attribute__((weak, alias("Default_Handler")))
void DEFAULT_HANDLER() Exception_Handler(void);
void DEFAULT_HANDLER() MemFault_Handler(void);
void DEFAULT_HANDLER() Timer0_Handler(void);
void DEFAULT_HANDLER() Timer1_Handler(void);
void DEFAULT_HANDLER() Message_Handler(void);
void DEFAULT_HANDLER() DMA0_Handler(void);
void DEFAULT_HANDLER() DMA1_Handler(void);
void DEFAULT_HANDLER() WAND_Handler(void);
void DEFAULT_HANDLER() SWI_Handler(void);

/* Default Handler */
void __attribute__((interrupt, disinterrupt)) Default_Handler(void)
{
	while(1) __asm__ volatile("TRAP 10");
}

/* C startup code */
void SystemInit(void)
{
	typedef void (*func)(void);
	extern func __preinit_array_start[], __preinit_array_end;
	extern func __init_array_start[], __init_array_end;
	extern func __ctors_start[], __ctors_end;
	extern func __dtors_start[], __dtors_end;
	extern func __fini_array_start[], __fini_array_end;
	extern int main();

	/* initialize CORE, DMA0, DMA1, MEM and MESH pointers */
	uint32_t coreid = MOVFS(COREID);
	CORE = (volatile struct CORE_s*)((coreid << 20) | 0xF0400);
	DMA0 = (volatile struct DMA_s* )((coreid << 20) | 0xF0500);
	DMA1 = (volatile struct DMA_s* )((coreid << 20) | 0xF0520);
	MEM  = (volatile struct MEM_s* )((coreid << 20) | 0xF0600);
	MESH = (volatile struct MESH_s*)((coreid << 20) | 0xF0700);

	/* call static constructors */
	for(int i = &__ctors_end - __ctors_start; i > 0; i--)
		__ctors_start[i-1]();
	for(int i = 0; i < &__preinit_array_end - __preinit_array_start; i++)
		__preinit_array_start[i]();
	for(int i = 0; i < &__init_array_end - __init_array_start; i++)
		__init_array_start[i]();

	/* call main() */
	main();

	/* call static destructors */
	for(int i = 0; i < &__fini_array_end - __fini_array_start; i++)
		__fini_array_start[i]();
	for(int i = 0; i < &__dtors_end - __dtors_start; i++)
		__dtors_start[i]();

	/* and never return */
	while(1) {
		__asm__ volatile("gid");
		__asm__ volatile("idle");
	}
}

/* Reset Handler */
__asm__ (
	/* declare function */
	".section .text\n"
	".global Reset_Handler\n"
	".type Reset_Handler, %function\n"
	".balign 8\n"
	"Reset_Handler:\n"

	/* set up stack pointer  */
	"    mov  sp, %low(_stack)\n"
	"    movt sp, %high(_stack)\n"

	/* set r43 and r63 to -1, in case of -m1reg-rXX */
	"mov   r0, 0\n"
	"sub   r43, r0, 1\n"
	"sub   r63, r0, 1\n"

	/* set initial fpu mode */
#if defined _MFP_MODE_INT
	"    mov   r0, %low(0x80000)\n"
	"    movt  r0, %high(0x80000)\n"
	"    movts CONFIG, r0\n"
#elif defined _MFP_MODE_TRUNCATE
	"    mov   r0, 1\n"
	"    movts CONFIG, r0\n"
#elif defined _MFP_MODE_ROUND
	"    mov   r0, 0\n"
	"    movts CONFIG, r0\n"
#else
	/* The default FPU mode is selected with a
	   compiler switch (-mfp_mode=xxx), but its
	   setting can not be detected here.
	   Define matching _MFP_MODE_XXX instead. */
	#error "No default FPU mode defined."
#endif

	/* mask all interrupts except SYNC */
	"mov   r0, #0xfffe\n"
	"movts imask, r0\n"

	/* return from interrupt to C startup code */
	"mov   r0, %low(SystemInit)\n"
	"movt  r0, %high(SystemInit)\n"
	"movts iret, r0\n"
	"rti\n"
);

/* Interrupt Vector Table */
__asm__ (
	".section .vectors\n"
	".global Vector_Table\n"
	"Vector_Table:\n"
	"    b.l Reset_Handler\n"
	"    b.l Exception_Handler\n"
	"    b.l MemFault_Handler\n"
	"    b.l Timer0_Handler\n"
	"    b.l Timer1_Handler\n"
	"    b.l Message_Handler\n"
	"    b.l DMA0_Handler\n"
	"    b.l DMA1_Handler\n"
	"    b.l WAND_Handler\n"
	"    b.l SWI_Handler\n"
	".section .text\n"
);

